#include <locale.h>
#include <libintl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <sectool.h>

void *xmalloc (size_t size) {
  void *ptr;
  
  ptr = malloc (size);
  if (ptr == NULL) {
    exit(E_FATAL);
  }
  
  return ptr;
}

void *xrealloc (void *orig, size_t size) {
  void *ptr;

  ptr = realloc (orig, size);
  if (size > 0 && ptr == NULL) {
    exit (E_FATAL);
  }
  
  return ptr;
}

void init_env (struct envinit_t e[]) {
  size_t i;
  
  for (i = 0; e[i].env_var != NULL; ++i)
    *(e[i].varptr) = getenv (e[i].env_var); 
  
  return;
}

void test_exit (int ret, const char *msg, ...) {
  va_list list;
  char *name = getenv ("NAME");
  
  if (name == NULL)
    name = "Unknown";
  
  switch (ret)
    {
    case E_OK:
      fprintf (stderr, "Test: %s, Succeeded\n", name);
      exit (ret);
      
    case E_FAIL:
    case E_FATAL:
      va_start (list, msg);
      fprintf (stderr, "Test: %s, Error: %d, - ", name, ret);
      vfprintf (stderr, msg, list);
      va_end (list);
      exit (ret);

    default:
      fprintf (stderr, _("Test: %s, Error: %d, - Internal error: Unknown return code %d\n"), name, E_FAIL, ret);
      exit (E_FAIL);
    }
}

void report (int type, int id, const char *msg, ...) {
  va_list list;
  char *fmt = NULL;

  switch (type)
    {
    case WARNING:
      fmt = xmalloc (sizeof (char) * (strlen (msg) + 6 + 1));
      sprintf (fmt, "W%02d : %s", id, msg);
      
      break;
    case ERROR:
      fmt = xmalloc (sizeof (char) * (strlen (msg) + 6 + 1));
      sprintf (fmt, "E%02d : %s", id, msg);
      
      break;
    case HINT:
      fmt = xmalloc (sizeof (char) * (strlen (msg) + 6 + 1));
      sprintf (fmt, "H%02d : %s", id, msg);
      
      break;
    case INFO:
      fmt = xmalloc (sizeof (char) * (strlen (msg) + 4 + 1));
      sprintf (fmt, "I : %s", msg);
      
      break;
    default:
      test_exit (E_FAIL, _("Internal error: Unknown message type %d\n"), type);
    }

  va_start (list, msg);
  vfprintf (stdout, fmt, list);
  va_end (list);
  free (fmt);

  return;
}
